% makeStars.m
% this script creates data to use for the various functions - it creates
% a vector named stars that contains 15 structures, where each structure 
% contains three fields that store the name, constellation, and 
% temperature of a star

names = {'Barnards Star' 'Epsilon Indi' 'Alioth' 'Regulus' 'Spica' 'Beta Crucis' ...
         'Rasalhague' 'Beta Indi' 'Alcor' 'Mizar' 'Denebola' 'Wolf 359' 'Ginan' ...
         'Beta Virginis' 'Gacrux'};
     
constellations = {'Ophiuchus' 'Indus' 'Ursa Major' 'Leo' 'Virgo' 'Crux' ...
         'Ophiuchus' 'Indus' 'Ursa Major' 'Ursa Major' 'Leo' 'Leo' 'Crux' ...
         'Virgo' 'Crux'};

temps = [3130  4280  9400  13260  22400  28200  8500  4380  8000  9000 ...
         8500  2800  4300  6100  3600];

stars = [];
for i = 1:15
    stars(i).name = names{i};
    stars(i).const = constellations{i};
    stars(i).temp = temps(i);
end

%--------------------------------------------------------------------------

function printAllStarsInfo(allStars)
% printAllInfo(allStars)
% prints the information about all of the stars contained in an input 
% vector of structures that contain fields for the name, constellation, 
% and temperature of each star 

for i = 1:length(allStars)
    disp(['name: ' allStars(i).name '   constellation: ' allStars(i).const ...
          '   temperature: ' num2str(allStars(i).temp)])
end

%--------------------------------------------------------------------------

function printTemp(allStars, starName, constellation)
% printTemp(allStars, starName, constellation)
% given an input vector of structures with information about stars, an
% input star name and constellation, prints the temperature of the input 
% star. The message 'star not found' is printed if the star is not found.
% Two solutions are given here, using a for loop and a while loop

% solution using a for loop and break
found = 0;          % will be set to 1 if star is found
for i = 1:length(allStars)
    if (strcmp(allStars(i).name, starName) & strcmp(allStars(i).const, constellation))
        % if star name and constellation are found, print temperature
        disp(['temperature of star: ' num2str(allStars(i).temp)]);
        found = 1;
        break
    end
end
if (found == 0)
    disp('star not found')
end

% solution using a while loop 
i = 1;
% while there are still more stars to check and the input star and 
% constellation have not veeb found, increment i (to keep searching)
while ((i <= length(allStars)) & ...
        ~(strcmp(allStars(i).name, starName) & strcmp(allStars(i).const, constellation)))
    i = i + 1;
end
% if the star and constellation were not found, then the while loop ended
% with the index i set to length(allStars)+1
if (i <= length(allStars))
    disp(['temperature of star: ' num2str(allStars(i).temp)])
else
    disp('star not found')
end

%--------------------------------------------------------------------------

function printStarsInConstellation(allStars,constellation)
% printStarsInConstellation(allStars, constellation)
% given an input vector of structures with information about stars, and an
% input constellation, prints the names of the stars that are located in 
% this constellation

disp(['stars in the constellation ' constellation])
found = 0;
for i = 1:length(allStars)
    if strcmp(allStars(i).const, constellation)
       disp(allStars(i).name)
       found = 1;
    end
end
if ~found
    disp(['constellation ' constellation ' not found'])
end

%--------------------------------------------------------------------------

function [hottestName, hottestTemp] = hottestStar(allStars)
% [name, temp] = hottestStar(allStars)
% given an input vector of structures with information about stars, returns
% the name and temperature of the hottest star overall

hottestTemp = 0;        % keeps track of highest temperature found so far
hottestName = '';       % name of hottest star found so far
for i = 1:length(allStars)
    if (allStars(i).temp > hottestTemp)
        % a hotter star is found, so update the hottest temp and name
        hottestTemp = allStars(i).temp;
        hottestName = allStars(i).name;
    end
end

%--------------------------------------------------------------------------

function [hottestName, hottestTemp] = hottestStarConst(allStars, constellation)
% [name, temp] = hottestStar(allStars, constellation)
% given an input vector of structures with information about stars and
% name of a constellation, return the name and temperature of the hottest 
% star in the constellation

hottestTemp = 0;        % keeps track of highest temperature found so far
hottestName = '';       % name of hottest star found so far in constellation
for i = 1:length(allStars)
    if strcmp(allStars(i).const, constellation)
        % found a star in the constellation, so check if it's hotter than
        % the hottest star found so far in this constellation
        if (allStars(i).temp > hottestTemp)
            hottestTemp = allStars(i).temp;
            hottestName = allStars(i).name;
        end
    end
end
if (hottestTemp == 0)
    disp('constellation not found')
end

%--------------------------------------------------------------------------

% testStars.m
% script to test all the functions that use a vector of structures with
% information about a set of stars

% run the makeStars.m script to create a vector of 15 structures, where
% each structure stores the name, constellation, and temperature of a
% star, in three fields called name, const, and temp

makeStars

% print information about all the stars

printAllStarsInfo(stars)
disp(' ')

% test function that prints the temperature of the star with the input
% name and constellation

printTemp(stars, 'Spica', 'Virgo')
disp(' ')

printTemp(stars, 'Mizar', 'Ursa Major')
disp(' ')

printTemp(stars, 'Spica', 'Leo')
disp(' ')

printTemp(stars, 'Ellen', 'Ursa Major')
disp(' ')

% test function that prints the names of stars in the input constellation

printStarsInConstellation(stars, 'Crux')
disp(' ')

printStarsInConstellation(stars, 'Ursa Major')
disp(' ')

printStarsInConstellation(stars, 'Stella')
disp(' ')

% test function that returns the name and temperature of the hottest star

[hotName1, hotTemp1] = hottestStar(stars)
disp(' ')

% test function that returns the name and temperature of the hottest star
% in the specified constellation

[hotName2, hotTemp2] = hottestStarConst(stars, 'Leo')
disp(' ')

[hitName3, hotTemp3] = hottestStarConst(stars, 'Stella');
disp(' ')


